/*
 *  This file is part of Autocomplete-Server
 *
 *  Autocomplete-Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Foobar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



/*===============================  HEADER  ==================================*/

#include "server.hpp"
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>
#include "util.hpp"

using namespace std;

/*---------------------------------------------------------------------------*/




/*============================  EMBEDDEDSERVER  =============================*/


EmbeddedServer::EmbeddedServer(std::string filename, int port){
	this->task.load(filename);
	this->daemon =  MHD_start_daemon(
		MHD_USE_THREAD_PER_CONNECTION,
		port,
		NULL,
		NULL,
		&EmbeddedServer::ahc_echo,
		(void*)this,
		MHD_OPTION_END
	);
}


EmbeddedServer::~EmbeddedServer(){
	if (this->daemon)
		MHD_stop_daemon(this->daemon);
	this->daemon = NULL;
}


struct MHD_Response* EmbeddedServer::sendFile(std::string filename){
	FILE* fd = fopen(filename.c_str(),"r");
	struct stat st;
	stat(filename.c_str(), &st);
	uint64_t size = st.st_size;
	return MHD_create_response_from_fd(size,fileno(fd));
}


int EmbeddedServer::ahc_echo(
	void* _server,
	struct MHD_Connection* connection,
	const char* url,
	const char* method,
	const char* version,
	const char* upload_data,
	size_t*     upload_data_size,
	void**      ptr
) {
	static int dummy;
	if (0 != strcmp(method, "GET"))
		return MHD_NO;
	if (&dummy != *ptr){
		*ptr = &dummy;
		return MHD_YES;
	}
	if (0 != *upload_data_size)
		return MHD_NO;

	string data;
	EmbeddedServer* server = (EmbeddedServer*) _server;
	int code  = MHD_HTTP_OK;
	struct MHD_Response* response;


	clock_t begin = clock();

	const char* query = MHD_lookup_connection_value(connection, MHD_GET_ARGUMENT_KIND, "q");
	if ( query )
		data = server->task.find( query );
	else
		data = "[]";
	response = MHD_create_response_from_data(data.size(),(void*) data.c_str(), MHD_NO,true);

	clock_t end = clock();
	double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
	printf("[%s - %s]: %f\n", method, url, elapsed_secs );


	int ret = MHD_queue_response(connection,code,response);
	MHD_destroy_response(response);
	return ret;
}


/*---------------------------------------------------------------------------*/
